{% extends "comments/base.html" %}
{% load i18n %}
{% load comments_ink %}

{% block title %}{% trans "Flag comment" %}{% endblock %}

{% block content %}
<main>
  <article>
    <div class="container">
      <h2 class="text-center">{% trans "Flag this comment?" %}</h2>
      <h6 class="text-center pb24">
        {% trans "Comment sent to:" %}<br/>
        <a href="{{ comment.content_object.get_absolute_url }}">{{ comment.content_object }}</a>
      </h6>
    </div>
    <div class="central-column">
      <div class="dci pb32">
        <div id="c{{ comment.id }}" class="comment-box">
          <div class="comment">
            <div class="header">
              <div>{% if comment.url and not comment.is_removed %}<a href="{{ comment.url }}" target="_new">{{ comment.name }}</a>{% else %}{{ comment.name }}{% endif %}&nbsp;&sdot;&nbsp;<span class="muted">{{ comment.submit_date }}</span>&nbsp;&nbsp;<a class="permalink" title="comment permalink" href="{% get_inkcomment_permalink comment page_number %}">¶</a></div>
            </div>
            <div class="body body-bordered">
              {{ comment.comment|linebreaks|escape }}
            </div>
            <div class="feedback">
              <div class="reactions">
                {% include "comments/comment_reactions.html" %}
              </div>
            </div>
          </div>
        </div>

        <section>
          <h6 class="text-center">{% trans "Click on the flag button to mark the following comment as inappropriate." %}</h6>
          <form action="." method="post">
            {% csrf_token %}
            <input type="hidden" name="next" value="{{ comment.get_absolute_url }}"/>
            <div class="inline-centered text-center">
              <button type="submit" class="primary" name="submit" value="flag">
                {% trans "Flag" %}
              </button>&nbsp;
              <a class="btn secondary" href="{{ comment.get_absolute_url }}">
                {% trans "cancel" %}
              </a>
            </div>
          </form>
        </section>
      </div>
    </div>
  </article>
</main>
{% endblock %}
