{% load i18n %}
{% load comments %}


<div style="display: none">
  <section class="reply-form" data-dci="reply-form-template">
    {% include "comments/reply_form.html" %}

    <div class="reply-form" data-dci="reply-textarea">
      <textarea rows="1" placeholder="{% translate 'Your reply' %}"></textarea>
    </div>
  </section>
</div>
